#!/usr/bin/python

# RPM package Summary and Description extraction for specspo
# Usage: package_extract [<dir>] [-o <file.po>]
# Version 1.1 from 17.06.2007
# (c) Andrey Cherepanov <cas@altlinux.ru> 2007
# (c) Evgenii Terechkov <evg@altlinux.ru> 2007

# Import modules
import sys, os, time

# Workaround for missed popen2 on int1
# need to install python-modules
#sys.path.append('/home/cas')
import popen2

po_file = "packages.po"

class POString:
	"""Class for put string in gettext translation file"""
	dict = {}
	
	def read( self, command, keepenv=False ):
		"""Read output of command. Keep current locale settings if second arg in true (default is false)"""
                oldlang = os.getenv("LANG")
                if not oldlang: oldlang = os.getenv("LC_ALL")
                if not keepenv: os.putenv("LANG", "C")

		r, w, e = popen2.popen3( command )
		s = r.read()
		r.close()
		e.close()
		w.close()

                if oldlang: os.putenv("LANG", oldlang)
		return s

        def gettrans(self, item, filename):
                """Get translation for item from filename or empty string, if there no one"""

                untrans = self.read( "rpm -qp --queryformat %%{%s} " % item + filename )
                trans = self.read( "rpm -qp --queryformat %%{%s} " % item + filename , True)
                if untrans != trans: return trans
                else: return ""

	def put( self, filename ):
		"""Put string to dict"""
		fn = os.path.basename( filename )

                for tag in "SUMMARY","DESCRIPTION":
                
                        key = self.read( "rpm -qp --queryformat %%{%s} " % tag + filename )
                        translation = self.gettrans(tag, filename)
                        if self.dict.has_key( key ):
                                self.dict[key][1] += ("# %s of " % tag.capitalize() + fn + chr(10))
                        else:
                                self.dict[key] = [ translation, "# %s of " % tag.capitalize() + fn + chr(10)]

	def dump( self, file ):
		"""Dump dict to file"""
		file.write('# RPM Packages translation file for Summary and Description\n')
		file.write('msgid ""\n')
		file.write('msgstr ""\n')
		file.write('"Project-Id-Version: packages-info\\n"\n')
		tm = time.strftime("%Y-%m-%d %H:%M+0400")
		file.write('"POT-Creation-Date: '+tm+'\\n"\n')
		file.write('"PO-Revision-Date: '+tm+'\\n"\n')
		file.write('"Last-Translator: NAME <EMAIL>\\n"\n')
		file.write('"Language-Team: TEAM\\n"\n')
		file.write('"MIME-Version: 1.0\\n"\n')
		file.write('"Content-Type: text/plain; charset=UTF-8\\n"\n')
		file.write('"Content-Transfer-Encoding: 8bit\\n"\n\n')
		
		for key in self.dict.keys():
                        translation = self.dict [ key ][0]
			comment = self.dict[ key ][1]

                        a = key.replace('\\', '\\\\').replace('"', '\\"').splitlines()
			key = "msgid "
			for ps in a:
				key = key + '"' + ps + '\\n"\n'
			key = key[:-4] + '"\n'
			file.write( comment )
			file.write( key )

                        if translation:
                                b = translation.replace('\\', '\\\\').replace('"', '\\"').splitlines()
                                translation = "msgstr "
                                for ps in b:
                                        translation = translation + '"' + ps + '\\n"\n'
                                translation = translation[:-4] + '"\n\n'
                        else:
                                translation ='msgstr ""\n\n'

                        file.write(translation)

# Path
if len(sys.argv) > 1 and sys.argv[1] != "-h":
	path = sys.argv[1]
	if len(sys.argv) > 3:
		po_file = sys.argv[3]
else:
	print "Usage: package_extract [<dir>] [-o <file.po>]"
	sys.exit()

# Build file list
dirs = path.split(",")
files = []
for directory in dirs:
	listing = os.listdir( directory )
	listing = [ directory + os.sep + x for x in listing ]
	files.extend( listing )

n = POString()

try:
	i = 1
	po = open( po_file, "w" )
	for fname in files:
		if fname[-4:] == ".rpm":
			n.put( fname )
			print "%6d %s" % ( i, os.path.basename( fname ) )
			i = i + 1
	n.dump( po )
	po.close()

except IOError, (errno, strerror):
    print "Error write to file '%s': %s" % ( po_file, strerror )
except:
    print "Unexpected error:", sys.exc_info()[0]
    raise	